@@ -21,6 +21,10 @@ def test_workflows():
 
     assert len(w._tasks.keys()) == 3
 
+    w.clear()
+
+    assert len(w._tasks.keys()) == 0
+
 
 def test_with_viewer(make_napari_viewer):
     viewer = make_napari_viewer()
@@ -40,13 +44,14 @@ def test_with_viewer(make_napari_viewer):
     image_layer = viewer.add_image(image)
 
     def segment(image:napari.types.ImageData, sigma:float=5)->napari.types.LabelsData:
+        print("segment:", image is image_layer.data)
         return image > 0.5
 
     def refine(image:napari.types.LabelsData) -> napari.types.LabelsData:
         return image
 
     from napari_workflows import WorkflowManager
-    manager = WorkflowManager.install(viewer)
+    manager = WorkflowManager.install(viewer, _for_testing=True)
 
     from napari_tools_menu import make_gui
     gui = make_gui(segment, viewer, auto_call=True)
@@ -60,6 +65,7 @@ def test_with_viewer(make_napari_viewer):
     print(viewer.layers)
 
     test_key_root = image_layer.name
+    print(workflow.followers_of(test_key_root))
     test_key = workflow.followers_of(test_key_root)[0]
 
     # test analysing workflow
@@ -85,13 +91,15 @@ def test_with_viewer(make_napari_viewer):
     assert len(workflow.leafs()) == 1
 
     # test manager
+    from napari_workflows._undo_redo_functionality import copy_workflow_state
     manager.update(list(viewer.layers)[1], segment, "Image", 2)
+    undo_state_check = copy_workflow_state(manager.workflow)
     manager._update_invalid_layer()
 
     # test code generation
     code = manager.to_python_code()
     print(code)
-    assert 14 < len(code.split("\n")) < 18
+    assert len(code.split("\n")) == 19
 
     # test event handling
     image_layer.data = image
@@ -114,6 +122,14 @@ def test_with_viewer(make_napari_viewer):
     _break_down_4d_to_2d_args([np.asarray([[0,1],[5,6]]), "hello", 3, 4], 0, viewer)
     _break_down_4d_to_2d_kwargs({'data':np.asarray([[0,1],[5,6]]), 'name': "hello", 'num1': 3, 'num2':4}, 0, viewer)
 
+    # check undo and redo
+    redo_state_check = copy_workflow_state(manager.workflow)
+    undone = manager.undo_redo_controller.undo()
+    assert undo_state_check._tasks == undone._tasks
+    
+    redone = manager.undo_redo_controller.redo()
+    assert redone._tasks == redo_state_check._tasks
+
     # test removing a layer
     viewer.layers.remove(image_layer)
 
@@ -129,6 +145,9 @@ def test_with_viewer(make_napari_viewer):
     workflow.remove_all_except(test_key)
     assert len(workflow._tasks.keys()) == 0
 
+    # running the remove zombies function #TODO make better test
+    manager.remove_zombies()
+
 
 def test_save_load():
     from napari_workflows import Workflow
